@@ -1,22 +1,28 @@
 /*
- * libheif thumbnailer for Gnome desktop.
- * Copyright (c) 2018 struktur AG, Dirk Farin <farin@struktur.de>
- *
- * This file is part of convert, an example application using libheif.
- *
- * convert is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
- *
- * convert is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with convert.  If not, see <http://www.gnu.org/licenses/>.
- */
+  libheif thumbnailer for Gnome desktop.
+
+  MIT License
+
+  Copyright (c) 2018 struktur AG, Dirk Farin <farin@struktur.de>
+
+  Permission is hereby granted, free of charge, to any person obtaining a copy
+  of this software and associated documentation files (the "Software"), to deal
+  in the Software without restriction, including without limitation the rights
+  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+  copies of the Software, and to permit persons to whom the Software is
+  furnished to do so, subject to the following conditions:
+
+  The above copyright notice and this permission notice shall be included in all
+  copies or substantial portions of the Software.
+
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+  SOFTWARE.
+*/
 #if defined(HAVE_CONFIG_H)
 #include "config.h"
 #endif
@@ -122,11 +128,13 @@ int main(int argc, char** argv)
 
   std::unique_ptr<Encoder> encoder(new PngEncoder());
 
+  int bit_depth=8; // TODO
+
   struct heif_image* image = NULL;
   err = heif_decode_image(image_handle,
                           &image,
                           encoder->colorspace(false),
-                          encoder->chroma(false),
+                          encoder->chroma(false, bit_depth),
                           NULL);
   if (err.code) {
     std::cerr << "Could not decode HEIF image : " << err.message << "\n";
@@ -148,9 +156,11 @@ int main(int argc, char** argv)
       thumbnail_height = input_height * size/input_width;
       thumbnail_width  = size;
     }
-    else {
+    else if (input_height > 0) {
       thumbnail_width  = input_width * size/input_height;
       thumbnail_height = size;
+    } else {
+      thumbnail_width  = thumbnail_height = 0;
     }
 
 
