@@ -25,12 +25,9 @@
 #include "config.h"
 #endif
 
-#if defined(HAVE_INTTYPES_H)
-#include <inttypes.h>
-#endif
-#if defined(HAVE_STDDEF_H)
-#include <stddef.h>
-#endif
+#include <cinttypes>
+#include <cstddef>
+
 #include <vector>
 #include <string>
 #include <memory>
@@ -208,7 +205,9 @@ namespace heif {
 
     int get_nesting_level() const { return m_nesting_level; }
 
-  private:
+    int64_t get_remaining_bytes() const { return m_remaining; }
+
+private:
     std::shared_ptr<StreamReader> m_istr;
     BitstreamRange* m_parent_range = nullptr;
     int m_nesting_level = 0;
