@@ -25,12 +25,9 @@
 #include "config.h"
 #endif
 
-#if defined(HAVE_INTTYPES_H)
-#include <inttypes.h>
-#endif
-#if defined(HAVE_STDDEF_H)
-#include <stddef.h>
-#endif
+#include <cinttypes>
+#include <cstddef>
+
 #include <vector>
 #include <string>
 #include <memory>
@@ -66,26 +63,28 @@ namespace heif {
   class Fraction {
   public:
     Fraction() { }
-  Fraction(int num,int den) : numerator(num), denominator(den) { }
+    Fraction(int32_t num,int32_t den);
 
     Fraction operator+(const Fraction&) const;
     Fraction operator-(const Fraction&) const;
     Fraction operator-(int) const;
     Fraction operator/(int) const;
 
-    int round_down() const;
-    int round_up() const;
-    int round() const;
+    int32_t round_down() const;
+    int32_t round_up() const;
+    int32_t round() const;
 
-    int numerator = 0;
-    int denominator = 1;
+    bool is_valid() const;
+
+    int32_t numerator = 0;
+    int32_t denominator = 1;
   };
 
 
   class BoxHeader {
   public:
     BoxHeader();
-    ~BoxHeader() { }
+    virtual ~BoxHeader() { }
 
     constexpr static uint64_t size_until_end_of_file = 0;
 
@@ -148,8 +147,7 @@ namespace heif {
   class Box : public BoxHeader {
   public:
     Box() { }
-  Box(const BoxHeader& hdr) : BoxHeader(hdr) { }
-    virtual ~Box() { }
+    Box(const BoxHeader& hdr) : BoxHeader(hdr) { }
 
     static Error read(BitstreamRange& range, std::shared_ptr<heif::Box>* box);
 
@@ -290,7 +288,7 @@ namespace heif {
     };
 
     struct Item {
-      heif_item_id item_ID;
+      heif_item_id item_ID = 0;
       uint8_t  construction_method = 0; // >= version 1
       uint16_t data_reference_index = 0;
       uint64_t base_offset = 0;
