@@ -67,6 +67,11 @@ namespace heif {
 
     void set_max_decoding_threads(int max_threads) { m_max_decoding_threads = max_threads; }
 
+    void set_maximum_image_size_limit(int maximum_size) {
+      m_maximum_image_width_limit = maximum_size;
+      m_maximum_image_height_limit = maximum_size;
+    }
+
     Error read(std::shared_ptr<StreamReader> reader);
     Error read_from_file(const char* input_filename);
     Error read_from_memory(const void* data, size_t size, bool copy);
@@ -240,6 +245,9 @@ namespace heif {
 
     Error add_XMP_metadata(std::shared_ptr<Image> master_image, const void* data, int size);
 
+    Error add_generic_metadata(std::shared_ptr<Image> master_image, const void* data, int size,
+                               const char* item_type, const char* content_type);
+
     void write(StreamWriter& writer);
 
   private:
@@ -259,6 +267,9 @@ namespace heif {
 
     int m_max_decoding_threads = 4;
 
+    uint32_t m_maximum_image_width_limit;
+    uint32_t m_maximum_image_height_limit;
+
     Error interpret_heif_file();
 
     void remove_top_level_image(std::shared_ptr<Image> image);
