@@ -221,6 +221,10 @@ EMSCRIPTEN_BINDINGS(libheif) {
     .value("heif_suberror_No_or_invalid_primary_item",heif_suberror_No_or_invalid_primary_item)
     .value("heif_suberror_No_infe_box",heif_suberror_No_infe_box)
     .value("heif_suberror_Security_limit_exceeded",heif_suberror_Security_limit_exceeded)
+    .value("heif_suberror_Unknown_color_profile_type",heif_suberror_Unknown_color_profile_type)
+    .value("heif_suberror_Wrong_tile_image_chroma_format",heif_suberror_Wrong_tile_image_chroma_format)
+    .value("heif_suberror_Invalid_fractional_number",heif_suberror_Invalid_fractional_number)
+    .value("heif_suberror_Invalid_image_size",heif_suberror_Invalid_image_size)
     .value("heif_suberror_Nonexisting_item_referenced",heif_suberror_Nonexisting_item_referenced)
     .value("heif_suberror_Null_pointer_argument",heif_suberror_Null_pointer_argument)
     .value("heif_suberror_Nonexisting_image_channel_referenced",heif_suberror_Nonexisting_image_channel_referenced)
@@ -233,6 +237,7 @@ EMSCRIPTEN_BINDINGS(libheif) {
     .value("heif_suberror_Unsupported_data_version",heif_suberror_Unsupported_data_version)
     .value("heif_suberror_Unsupported_color_conversion",heif_suberror_Unsupported_color_conversion)
     .value("heif_suberror_Unsupported_item_construction_method",heif_suberror_Unsupported_item_construction_method)
+    .value("heif_suberror_Unsupported_bit_depth",heif_suberror_Unsupported_bit_depth)
     ;
   emscripten::enum_<heif_compression_format>("heif_compression_format")
     .value("heif_compression_undefined", heif_compression_undefined)
@@ -248,6 +253,10 @@ EMSCRIPTEN_BINDINGS(libheif) {
     .value("heif_chroma_444", heif_chroma_444)
     .value("heif_chroma_interleaved_RGB", heif_chroma_interleaved_RGB)
     .value("heif_chroma_interleaved_RGBA", heif_chroma_interleaved_RGBA)
+    .value("heif_chroma_interleaved_RRGGBB_BE", heif_chroma_interleaved_RRGGBB_BE)
+    .value("heif_chroma_interleaved_RRGGBBAA_BE", heif_chroma_interleaved_RRGGBBAA_BE)
+    .value("heif_chroma_interleaved_RRGGBB_LE", heif_chroma_interleaved_RRGGBB_LE)
+    .value("heif_chroma_interleaved_RRGGBBAA_LE", heif_chroma_interleaved_RRGGBBAA_LE)
     // Aliases
     .value("heif_chroma_interleaved_24bit", heif_chroma_interleaved_24bit)
     .value("heif_chroma_interleaved_32bit", heif_chroma_interleaved_32bit)
